/**
 * Make directories.
 * This is an apeman task contribution plugin.
 * @module apeman-task-contrib-mkdir
 * @see https://github.com/apeman-repo/apeman
 */

"use strict";

var async = require('async'),
    mkdirp = require('mkdirp'),
    fs = require('fs');

/**
 * Task worker function to make directories.
 * @function mkdir
 * @param {TaskWorkerContext} $context - Task context.
 * @param {TaskWorkerParams} $params - Task parameters.
 * @param {TaskWorkerCallback} callback - Callback when done.
 */
function mkdir($context, $params, callback) {
    var logger = $context.logger;

    $params = _parseParams($params);
    _rejectExisting($params.dirnames, function (dirnames) {
        async.eachSeries(dirnames, function (dirname, callback) {
            mkdirp(dirname, function (err) {
                if (!err) {
                    logger.debug('Directory generated: ' + dirname);
                }
                callback(err);
            });
        }, callback);
    });
}

function _parseParams($params) {
    if (Array.isArray($params)) {
        $params = {dirnames: $params};
    }
    $params.dirnames = [].concat($params.dirnames || []);
    return $params;
}

function _rejectExisting(dirnames, callback) {
    async.reject(dirnames, fs.exists, callback);
}


module.exports = mkdir;
